import { ApiPromise, ApiRequestOptions, RetrieveBatchResponse } from "@trigger.dev/core/v3";
import { batchTriggerById, batchTriggerByIdAndWait, batchTriggerTasks, batchTriggerAndWaitTasks } from "./shared.js";
export declare const batch: {
    trigger: typeof batchTriggerById;
    triggerAndWait: typeof batchTriggerByIdAndWait;
    triggerByTask: typeof batchTriggerTasks;
    triggerByTaskAndWait: typeof batchTriggerAndWaitTasks;
    retrieve: typeof retrieveBatch;
};
/**
 * Retrieves details about a specific batch by its ID.
 *
 * @param {string} batchId - The unique identifier of the batch to retrieve
 * @param {ApiRequestOptions} [requestOptions] - Optional API request configuration options
 * @returns {ApiPromise<RetrieveBatchResponse>} A promise that resolves with the batch details
 *
 * @example
 * // First trigger a batch
 * const response = await batch.trigger([
 *   { id: "simple-task", payload: { message: "Hello, World!" } }
 * ]);
 *
 * // Then retrieve the batch details
 * const batchDetails = await batch.retrieve(response.batchId);
 * console.log("batch", batchDetails);
 */
declare function retrieveBatch(batchId: string, requestOptions?: ApiRequestOptions): ApiPromise<RetrieveBatchResponse>;
export {};
