import { batchTrigger, batchTriggerAndWait, createTask, createSchemaTask, createToolTask, SubtaskUnwrapError, trigger, triggerAndPoll, triggerAndWait } from "./shared.js";
export { SubtaskUnwrapError };
import type { AnyTask, BatchItem, BatchResult, BatchRunHandle, Queue, RunHandle, Task, TaskIdentifier, TaskOptions, TaskOutput, TaskPayload, TriggerOptions, TaskRunResult, TaskFromIdentifier } from "./shared.js";
export type { AnyTask, BatchItem, BatchResult, BatchRunHandle, Queue, RunHandle, Task, TaskIdentifier, TaskOptions, TaskOutput, TaskPayload, TriggerOptions, TaskRunResult, TaskFromIdentifier, };
/** Creates a task that can be triggered
 * @param options - Task options
 * @example
 *
 * ```ts
 * import { task } from "@trigger.dev/sdk/v3";
 *
 * export const helloWorld = task({
    id: "hello-world",
 *    run: async (payload: { url: string }) => {
 *    return { hello: "world" };
 *  },
 * });
 *
 * ```
 *
 * @returns A task that can be triggered
 */
export declare const task: typeof createTask;
export declare const schemaTask: typeof createSchemaTask;
export declare const toolTask: typeof createToolTask;
export declare const tasks: {
    trigger: typeof trigger;
    triggerAndPoll: typeof triggerAndPoll;
    batchTrigger: typeof batchTrigger;
    triggerAndWait: typeof triggerAndWait;
    batchTriggerAndWait: typeof batchTriggerAndWait;
};
